(function (wpI18n, wpBlocks, wpElement, wpEditor, wpComponents) {
    const {__} = wp.i18n;
    const {Component, Fragment} = wp.element;
    const {registerBlockType} = wpBlocks;
    const {BlockControls, BlockAlignmentToolbar} = wpEditor;
    const {Modal, FocusableIframe, IconButton} = wp.components;

    /*const el = wp.element.createElement;*/
    class WpmfGallery extends Component {
        constructor() {
            super(...arguments);
            this.state = {
                isOpen: false
            };

            this.openModal = this.openModal.bind(this);
            this.closeModal = this.closeModal.bind(this);
            this.addEventListener = this.addEventListener.bind(this);
            this.componentDidMount = this.componentDidMount.bind(this);
        }

        openModal() {
            if (!this.state.isOpen) {
                this.setState({isOpen: true});
            }
        }

        closeModal() {
            if (this.state.isOpen) {
                this.setState({isOpen: false});
            }
        }

        addEventListener(e) {
            if (!e.data.hasfiles) {
                return;
            }

            if (e.data.type !== 'wpmfgalleryinsert') {
                return;
            }

            if (e.data.idblock !== this.props.id) {
                return;
            }

            this.setState({
                isOpen: false
            });
            
            const {setAttributes} = this.props;
            setAttributes({
                html: e.data.html,
                hasfiles: e.data.hasfiles
            });
        }

        componentDidMount() {
            window.addEventListener("message", this.addEventListener, false);
        }

        render() {
            const {attributes, setAttributes} = this.props;
            const {
                align,
                html,
                hasfiles
            } = attributes;
            const renderHTML = (rawHTML: string) => React.createElement("div", { dangerouslySetInnerHTML: { __html: rawHTML } });
            return (
                <Fragment>
                    {hasfiles && (
                        <BlockControls>
                            <BlockAlignmentToolbar value={ align } onChange={ ( align ) => setAttributes( { align: align } ) } />
                            <IconButton
                                className="components-toolbar__control"
                                label={ wpmfgalleryblocks.l18n.edit }
                                icon={ 'edit' }
                                onClick={ () => this.setState( { isOpen: true } ) }
                            />

                            <IconButton
                                className="components-toolbar__control"
                                label={ wpmfgalleryblocks.l18n.remove }
                                icon={ 'no' }
                                onClick={ () => setAttributes( { hasfiles: false, 'html': '' } ) }
                            />
                        </BlockControls>
                    ) }

                    {(hasfiles) &&
                    renderHTML(html)
                    }
                    {!hasfiles &&
                    <button className="components-button is-button is-default is-large aligncenter"
                            onClick={this.openModal}>{wpmfgalleryblocks.l18n.btnopen}</button>}
                    {this.state.isOpen ?
                        <Modal
                            title={wpmfgalleryblocks.l18n.gallery_title}
                            onSelect={(image) => setAttributes({html: image.url, imageID: image.id})}
                            onRequestClose={this.closeModal}>
                            <FocusableIframe
                                src={wpmfgalleryblocks.vars.admin_gallery_page + '&idblock=' + this.props.id}
                            />
                        </Modal>
                        : null}
                </Fragment>
            );
        }
    }

    registerBlockType('wpmf/block-gallery', {
        title: wpmfgalleryblocks.l18n.gallery_title,
        icon: 'format-gallery',
        category: 'common',
        keywords: [
            __('gallery'),
            __('file')
        ],
        attributes: {
            hasfiles: {
                type: 'string',
                default: false
            },
            html: {
                type: 'string',
                default: ''
            },
            align: {
                type: 'string',
                default: 'center'
            }
        },
        edit: WpmfGallery,
        save: ({attributes}) => {

            const {
                align,
                html,
                hasfiles
            } = attributes;
            const renderHTML = (rawHTML: string) => React.createElement("div", { dangerouslySetInnerHTML: { __html: rawHTML } });
            return (
                (hasfiles) &&
                    renderHTML(html)
            );
        },
        getEditWrapperProps( attributes ) {
            const { align } = attributes;
            const props = { 'data-resized': true };

            if ( 'left' === align || 'right' === align || 'center' === align ) {
                props[ 'data-align' ] = align;
            }

            return props;
        }
    });
})(wp.i18n, wp.blocks, wp.element, wp.editor, wp.components);